package dev.flyfish.framework.r2dbc.config.mapping;

import dev.flyfish.framework.utils.JacksonUtil;
import org.springframework.data.convert.CustomConversions;
import org.springframework.data.mapping.IdentifierAccessor;
import org.springframework.data.mapping.PersistentPropertyAccessor;
import org.springframework.data.mapping.context.MappingContext;
import org.springframework.data.r2dbc.convert.MappingR2dbcConverter;
import org.springframework.data.r2dbc.mapping.OutboundRow;
import org.springframework.data.relational.core.mapping.RelationalPersistentEntity;
import org.springframework.data.relational.core.mapping.RelationalPersistentProperty;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.r2dbc.core.Parameter;
import org.springframework.util.ClassUtils;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.Collections;
import java.util.Optional;

/**
 * 全功能支持的转换器
 *
 * @author wangyu
 * 增加json对象的支持
 */
@SuppressWarnings("deprecation")
public class FullMappingR2dbcConverter extends MappingR2dbcConverter {

    public FullMappingR2dbcConverter(MappingContext<? extends RelationalPersistentEntity<?>, ? extends RelationalPersistentProperty> context, CustomConversions conversions) {
        super(context, conversions);
    }

    // ----------------------------------
    // Entity writing
    // ----------------------------------

    @Override
    public void write(Object source, OutboundRow sink) {

        Class<?> userClass = ClassUtils.getUserClass(source);

        Optional<Class<?>> customTarget = getConversions().getCustomWriteTarget(userClass, OutboundRow.class);
        if (customTarget.isPresent()) {

            OutboundRow result = getConversionService().convert(source, OutboundRow.class);
            sink.putAll(result);
            return;
        }

        writeInternal(source, sink, userClass);
    }

    private void writeInternal(Object source, OutboundRow sink, Class<?> userClass) {

        RelationalPersistentEntity<?> entity = getMappingContext().getRequiredPersistentEntity(userClass);
        PersistentPropertyAccessor<?> propertyAccessor = entity.getPropertyAccessor(source);

        writeProperties(sink, entity, propertyAccessor, entity.isNew(source));
    }

    private void writeProperties(OutboundRow sink, RelationalPersistentEntity<?> entity,
                                 PersistentPropertyAccessor<?> accessor, boolean isNew) {

        for (RelationalPersistentProperty property : entity) {

            if (!property.isWritable()) {
                continue;
            }

            Object value;

            if (property.isIdProperty()) {
                IdentifierAccessor identifierAccessor = entity.getIdentifierAccessor(accessor.getBean());
                value = identifierAccessor.getIdentifier();
            } else {
                value = accessor.getProperty(property);
            }

            if (value == null) {
                writeNullInternal(sink, property);
                continue;
            }

            if (getConversions().isSimpleType(value.getClass())) {
                writeSimpleInternal(sink, value, isNew, property);
            } else {
                writePropertyInternal(sink, value, isNew, property);
            }
        }
    }

    private void writeNullInternal(OutboundRow sink, RelationalPersistentProperty property) {

        sink.put(property.getColumnName(), Parameter.empty(getPotentiallyConvertedSimpleNullType(property.getType())));
    }

    private void writeSimpleInternal(OutboundRow sink, Object value, boolean isNew,
                                     RelationalPersistentProperty property) {

        Object result = getPotentiallyConvertedSimpleWrite(value, Object.class);

        sink.put(property.getColumnName(),
                Parameter.fromOrEmpty(result, getPotentiallyConvertedSimpleNullType(property.getType())));
    }

    private void writePropertyInternal(OutboundRow sink, Object value, boolean isNew,
                                       RelationalPersistentProperty property) {
        String result = JacksonUtil.toJson(value).orElse(null);

        // 直接以json填充，无论数组还是对象
        sink.put(property.getColumnName(), Parameter.fromOrEmpty(result, String.class));
    }


    /**
     * Returns given object as {@link Collection}. Will return the {@link Collection} as is if the source is a
     * {@link Collection} already, will convert an array into a {@link Collection} or simply create a single element
     * collection for everything else.
     *
     * @param source
     * @return
     */
    private static Collection<?> asCollection(Object source) {

        if (source instanceof Collection) {
            return (Collection<?>) source;
        }

        return source.getClass().isArray() ? CollectionUtils.arrayToList(source) : Collections.singleton(source);
    }

    /**
     * Checks whether we have a custom conversion registered for the given value into an arbitrary simple type. Returns
     * the converted value if so. If not, we perform special enum handling or simply return the value as is.
     *
     * @param value
     * @return
     */
    @Nullable
    private Object getPotentiallyConvertedSimpleWrite(@Nullable Object value, Class<?> typeHint) {

        if (value == null) {
            return null;
        }

        if (Object.class != typeHint) {

            if (getConversionService().canConvert(value.getClass(), typeHint)) {
                value = getConversionService().convert(value, typeHint);
            }
        }

        Optional<Class<?>> customTarget = getConversions().getCustomWriteTarget(value.getClass());

        if (customTarget.isPresent()) {
            return getConversionService().convert(value, customTarget.get());
        }

        return Enum.class.isAssignableFrom(value.getClass()) ? ((Enum<?>) value).name() : value;
    }

    private Object writeNestedObject(@NonNull Object object) {
        return JacksonUtil.toJson(object).orElse(null);
    }

    private Class<?> getPotentiallyConvertedSimpleNullType(Class<?> type) {

        Optional<Class<?>> customTarget = getConversions().getCustomWriteTarget(type);

        if (customTarget.isPresent()) {
            return customTarget.get();

        }

        if (type.isEnum()) {
            return String.class;
        }

        return type;
    }
}
