package dev.flyfish.framework.logging.service;

import dev.flyfish.framework.context.DateContext;
import dev.flyfish.framework.logging.config.LoggingTextRegistry;
import dev.flyfish.framework.logging.domain.Log;
import dev.flyfish.framework.logging.domain.LogType;
import dev.flyfish.framework.utils.JacksonUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.DisposableBean;

import java.util.Optional;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 日志管理器
 *
 * @author wangyu
 */
@Slf4j
public class LogManager implements DisposableBean {

    // 固定线程池
    private final ExecutorService executorService = Executors.newFixedThreadPool(20);
    @Resource
    private LogService logService;
    @Resource
    private LoggingTextRegistry registry;

    /**
     * 尝试记录日志
     *
     * @param context 上下文
     */
    public void tryLog(LogContext context) {
        executorService.execute(() -> {
            // 构建日志
            Log log = new Log();
            log.setType(LogType.OPERATION);
            log.setSuccess(context.isSuccess());
            bodyString(context.getArgs()).ifPresent(log::setBody);
            log.setModule(context.getModule());
            log.setBusiness(Optional.ofNullable(context.getBusiness())
                    .filter(StringUtils::isNotBlank)
                    .map(registry::text)
                    .orElse(context.getBusiness()));
            log.setError(Optional.ofNullable(context.getError()).flatMap(JacksonUtil::toJson).orElse(null));
            log.setResponse(Optional.ofNullable(context.getResult()).flatMap(JacksonUtil::toJson).orElse(null));
            log.setOperator(context.getUser());
            log.setSignature(context.getSignature());
            log.setPeriod(context.getEndTime() - context.getStartTime());
            log.setStartTime(DateContext.ofMillis(context.getStartTime()));
            log.setEndTime(DateContext.ofMillis(context.getEndTime()));
            // 先不拼接
            log.setUri(null);
            // 写入日志
            logService.create(log).subscribe();
        });
    }

    private Optional<String> bodyString(Object[] args) {
        if (ArrayUtils.isEmpty(args)) {
            return Optional.empty();
        }
        if (args.length == 1) {
            return JacksonUtil.toJson(args[0]);
        }
        return JacksonUtil.toJson(args);
    }

    @Override
    public void destroy() {
        if (!executorService.isShutdown()) {
            executorService.shutdownNow();
        }
    }
}
