package dev.flyfish.framework.user.config.converter;

import dev.flyfish.framework.user.config.RSAKeys;
import dev.flyfish.framework.user.config.captcha.CaptchaValidator;
import dev.flyfish.framework.user.config.properties.SecurityProperties;
import dev.flyfish.framework.user.utils.RSAUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.server.authentication.ServerAuthenticationConverter;
import org.springframework.util.MultiValueMap;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

/**
 * 超级简单的认证转换器
 *
 * @author wangyu
 */
@Slf4j
@RequiredArgsConstructor
public class EncryptedAuthenticationConverter implements ServerAuthenticationConverter {

    private final String usernameParameter = "username";

    private final String passwordParameter = "password";

    private final String tokenParameter = "token";

    private final SecurityProperties securityProperties;

    private final ObjectProvider<CaptchaValidator> validator;

    @Override
    public Mono<Authentication> convert(ServerWebExchange exchange) {
        return exchange.getFormData().map(this::createAuthentication);
    }

    private UsernamePasswordAuthenticationToken createAuthentication(MultiValueMap<String, String> data) {
        String username = data.getFirst(this.usernameParameter);
        String password = data.getFirst(this.passwordParameter);
        String token = data.getFirst(this.tokenParameter);
        validator.ifAvailable(v -> v.verify(token));
        if (securityProperties.isRsa()) {
            password = decrypt(password);
        }
        val at = new UsernamePasswordAuthenticationToken(username, password);
        at.setDetails(token);
        return at;
    }

    private String decrypt(String password) {
        try {
            return RSAUtils.decrypt(password, RSAKeys.PRIVATE_KEY);
        } catch (IllegalBlockSizeException | InvalidKeyException | BadPaddingException | NoSuchAlgorithmException |
                 NoSuchPaddingException e) {
            log.error("尝试解密密码出错", e);
            throw new IllegalArgumentException("非法请求！密码格式校验失败！");
        } catch (IllegalArgumentException e) {
            log.error("抛出参数异常", e);
            throw new IllegalArgumentException("密码未加密，请求无效！" + e.getMessage());
        }
    }
}
