package dev.flyfish.framework.user.service.impl;

import dev.flyfish.framework.domain.base.Qo;
import dev.flyfish.framework.domain.po.Department;
import dev.flyfish.framework.domain.tree.TreeDomain;
import dev.flyfish.framework.user.service.DepartmentService;
import dev.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.mongodb.core.ReactiveMongoOperations;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import jakarta.annotation.Resource;
import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 部门服务
 *
 * @author wangyu
 */
@Service
@Slf4j
@ConditionalOnProperty("spring.data.mongodb.host")
public class MongoDepartmentServiceImpl extends BaseReactiveServiceImpl<Department> implements DepartmentService {

    @Resource
    private ReactiveMongoOperations reactiveMongoOperations;

    /**
     * 获取子部门列表
     *
     * @return 结果
     */
    public Mono<Set<String>> getSubCodes(Set<String> parents) {
        Query query = Query.query(Criteria.where("parentIds").in(parents));
        query.fields().include("_id");
        return reactiveMongoOperations.find(query, Department.class).map(Department::getId).collect(Collectors.toSet());
    }

    /**
     * 查询列表
     *
     * @param query 查询
     * @return 结果
     */
    @Override
    public Flux<Department> getList(Qo<Department> query) {
        return DepartmentServiceHelper.convertQo(query, this::getMinDepthInIds).flatMapMany(super::getList);
    }

    /**
     * 在一批编码中找到深度最小的值
     *
     * @param codes 编码们
     * @return 结果
     */
    private Mono<Integer> getMinDepthInIds(Collection<String> codes) {
        return reactiveMongoOperations.aggregate(
                        Aggregation.newAggregation(
                                Aggregation.match(Criteria.where("_id").in(codes)),
                                Aggregation.group().min("depth").as("depth")),
                        Department.class, Department.class)
                .single()
                .map(TreeDomain::getDepth);
    }

}
