package dev.flyfish.framework.service;

import dev.flyfish.framework.bean.SyncVo;
import dev.flyfish.framework.domain.base.Domain;
import dev.flyfish.framework.domain.base.DomainService;
import dev.flyfish.framework.domain.base.Qo;
import org.springframework.data.domain.Page;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * 基础Service，全异步接口
 * 代理Repo的所有方法，内部封装常用的全部操作
 *
 * @author wybab
 */
public interface BaseReactiveService<T extends Domain> extends DomainService {

    /**
     * 查询
     *
     * @param query 查询模型
     * @return 结果
     */
    Mono<T> getOne(Qo<T> query);

    /**
     * 通过Id查询
     *
     * @param id 主键
     * @return 结果
     */
    Mono<T> getById(String id);

    /**
     * 通过实体名称访问
     *
     * @param name 名称
     * @return 结果
     */
    Mono<T> getByName(String name);

    /**
     * 根据ID集合来查询
     *
     * @param ids id列表
     * @return 结果
     */
    Flux<T> getByIds(List<String> ids);

    /**
     * 通过某个字段的集合包含查询
     *
     * @param key    键
     * @param values 值
     * @return 结果
     */
    <K> Flux<T> getByValues(String key, List<K> values);

    /**
     * 查询列表
     *
     * @param query 查询
     * @return 结果
     */
    Flux<T> getList(Qo<T> query);

    /**
     * 分页查询列表
     *
     * @param query 查询实体
     * @return 结果
     */
    Mono<Page<T>> getPageList(Qo<T> query);

    /**
     * 获取所有对象
     *
     * @return 结果
     */
    Flux<T> getAll();

    /**
     * 查询总记录数
     *
     * @return 总数
     */
    Mono<Long> countAll();

    /**
     * 查询总记录数
     *
     * @param query 查询实体
     * @return 结果
     */
    Mono<Long> count(Qo<T> query);

    /**
     * 查询是否存在
     *
     * @param query 查询
     * @return 结果
     */
    Mono<Boolean> exists(Qo<T> query);

    /**
     * 插入新记录
     *
     * @param entity 实体
     */
    Mono<T> create(T entity);

    /**
     * 插入 不插入null字段
     *
     * @param entity 实体
     */
    Mono<T> createSelective(T entity);

    /**
     * 删除
     *
     * @param entity 实体
     */
    Mono<Void> delete(T entity);

    /**
     * 根据Id删除
     *
     * @param id 主键
     */
    Mono<Void> deleteById(String id);

    /**
     * 根据id更新
     *
     * @param entity 实体
     */
    Mono<T> updateById(T entity);

    /**
     * 不update null
     *
     * @param entity 实体
     */
    Mono<T> updateSelectiveById(T entity);

    /**
     * 根据ID集合批量删除
     *
     * @param ids id结合
     */
    Mono<Void> deleteBatchByIds(List<String> ids);

    /**
     * 通过某一个键删除所有
     *
     * @param qo 查询实体
     */
    Mono<Void> deleteAll(Qo<T> qo);

    /**
     * 删除全部实体，危险！
     */
    Mono<Void> deleteAll();

    /**
     * 批量更新
     *
     * @param entities 要批量更新的集合
     */
    Flux<T> updateBatch(List<T> entities);

    /**
     * 同步数据
     *
     * @param entities 数据们
     * @return 结果
     */
    Mono<SyncVo<T>> sync(List<T> entities);
}
